#! /usr/bin/perl -w
use strict;
use Getopt::Long;
use vars qw($PATH);

BEGIN {
    use FindBin qw($Bin);
    use lib "$Bin";
    $PATH = $Bin;
}

use Tracklib;
use Time::localtime;

################################## Variables #############################################################################################

###########################################
#### Added by Matti Kankainen          ####
#### Voit taman avulla katsoa          ####
#### missa aikaa menee hukkaan         #### 
####				       ####
#### kutsu kayttamalla now = ctime();  ####
####				       ####
###########################################

my($secX,$minX,$hourX,$mdayX,$monX,$yearX,$wdayX,$ydayX,$isdstX) = localtime(time);
my($now) = ctime();

my %opt = ();
my %legend = (
              "seqfile1=s"               =>      "file containing the sequences in fasta format (combained_sequence_set1)",
              "outfile=s"                =>      "filename for saving the otput",
              "webfile=s"                =>      "filename for web-file",
              "background_organism=s"    =>      "parameter defining which organisms database is used as query db ",
              "sequence_length=s"         =>      "defines max length of the sequence",
              "Evalue=s"                 =>      "threshold E-value for selecting the hits",
              "search_pattern=s"         =>      "file containing search patterns",
	      "input_number=s"           =>      "number of clusters given",
              "pat_overlap=s"            =>      "overlapping",
	      "browser=s"                =>      "browser type in use"
	      );

GetOptions(\%opt, "help!", keys %legend);

my($output)                                      = "";                        # output folder
my($webput)                                      = "";                        # folder that contains the results in Zope
my(@files)                                       = ();                        # array containing input files
my($db0)                                         = "";                        # variable defining the organism
my($db)                                          = "";                        # variable defining blast database to use
my(%results)                                     = ();                        # hash containing query result
my($maxlength)                                   = 0;                         # max length of the sequence
my($Evalue)                                      = "";                        # E value threshold for blast result
my($fastafile)                                   = "";                        # File containing generated Fasta-file for Blast
my($hidden)                                      = "";                        # variable containing selected hits
my($selectedhit)                                 = "";                        # variable containing selected hits
my($amount_seq)                                  = 0;                         # total number of query sequences
my($patterns)                                    = "";                        # file containing search patterns for visualization tool
my($pattern)                                     = "";                        # contains patterns for the hidden variable
my(@sequences)                                   = ();                        # sequence from the file 
my($order)                                       = "";
my($counter)                                     = 0;
my($current_id)                                  = "None";
my($hits_shown)                                  = 10;                        # Number of hits shown per sequence
my($query_seq_counter)                           = 0;                         # order no of current query sequence under investigation
my(@patternarray)                                = ();                        # arrray containing patterns
my(@seqnames)                                    = ();                        # array containing names of the query sequences
my($nclu)                                        = 0;                         # number of clusters
my($first)                                       = 0;
my($browser)                                     = 2;
my($pat_overlap)                                 = 0;
my($print_variable)                              = "";
my($no_blast)                                    = 0;


if (exists($opt{'seqfile1'}))            { $files[0] = $opt{'seqfile1'}; }       else { die "Parameter missing: seqfile1\n"; }
if (exists($opt{'outfile'}))             { $output = $opt{'outfile'}; }          else { die "Parameter missing: outfile\n"; }
if (exists($opt{'webfile'}))             { $webput = $opt{'webfile'}; }          else { die "Parameter missing: webfile\n"; }
if (exists($opt{'background_organism'})) { $db0 = $opt{'background_organism'};}  else { die "Missing parameter: background_organism"; }
if (exists($opt{'sequence_length'}))     { $maxlength = $opt{'sequence_length'};} else { die "Missing parameter: sequence_length"; }
if (exists($opt{'Evalue'}))              { $Evalue = $opt{'Evalue'}; }           else { die "Missing parameter: Evalue"; }
if (exists($opt{'search_pattern'}))      { $patterns = $opt{'search_pattern'};}  else { die "Missing parameter: search_pattern"; }
if (exists($opt{'input_number'}))        { $nclu = $opt{'input_number'}; }       else { die "Missing parameter: input_number"; }
if (exists($opt{'pat_overlap'}))         { $pat_overlap = $opt{'pat_overlap'}; } else { die "Missing parameter: pat_overlap"; }
if (exists($opt{'browser'}))             { $browser = $opt{'browser'}; }         else { die "Missing parameter: browser"; }

$db="/data/databases/blast/tracker_blastdb/".$db0.".faa";

@patternarray=&ReadModify($patterns);
my($no_pats)=scalar@patternarray;

for(my ($s)=0;$s<($no_pats);$s++){
    $pattern=$pattern.$patternarray[$s]."\n";
}


####################################################################################################################################################
# The fasta file containing multiple sequences could be used as an input as such for blastall program. But because the length of the  each sequence
# is wanted to be checked the seqfile is readen and each sequence is being shortened if necessary. 
# Reading in sequences from the file(s):

my ($no_files)=scalar@files;
my ($s)=0;

for(my ($r)=0;$r<($no_files);$r++){
    $fastafile="$PATH/fasta.txt";
    &clean_fastafile($fastafile);                                              #makes sure the fasta file used is empty before usage
    @seqnames=@{&ReadClusters($files[$r], $maxlength,$fastafile,$r)};          #when several clusters are given as input, contains markings #clustern
    for($s=0;$s<(scalar@seqnames);$s++){
       if($seqnames[$s]!~m/^#cluster.*/){
	  $amount_seq++;
       }
    }
 }

######################################################################################################################################################### Printing the Blast result into the html-page according to the hash information
#### Printing order is based on original order of query sequences
#### The assumption that each query sequence has atleast one  blast hit is tested. If this is not true, the info is printed for the user 
# Weakness of this option is that the knowledge of hash structure is needed

#$now = ctime();
#print STDOUT "$now\n<br>";

%results=%{&blast_search($fastafile,$db, $Evalue, $output)};

my($Query_ID, $Subject_ID, $pros_identity, $align_le, $mismatches, $gap_open, $q_start, $q_end, $s_start, $s_end, $E_value, $bit_score,$order_no);

#$now = ctime();
#print STDOUT "$now\n";

print STDOUT "<br>\n";
$hidden="";
print STDOUT "<form name=\"tracker\" method=\"post\" enctype=\"multipart/form-data\">\n";
print STDOUT "\t <table border='1' cellpadding='4' cellspacing='0' align='center' bordercolor='#00008B' bgcolor='#B0C4DE' width='95%' colspan='12'>\n ";
print STDOUT "\t\t <tr bgcolor='#B0C4DE'>\n ";
print STDOUT "\t\t\t <td>\n ";
print STDOUT "\t\t\t\t <center>Blast Results </center>\n";

my ($i)=0;

for $order (sort {$a <=> $b} (keys %results) ) {
    if($order==1){
	$current_id="None";
    }
    $Query_ID=$results{$order}{'Query_ID'};
    $Subject_ID=$results{$order}{ 'Subject_ID' };
    $pros_identity=$results{$order}{ 'pros_identity' };
    $align_le=$results{$order}{ 'align_le' };
    $mismatches=$results{$order}{ 'mismatches' };
    $gap_open=$results{$order}{ 'gap_open' };
    $q_start=$results{$order}{ 'q_start' };
    $q_end=$results{$order}{ 'q_end' };
    $s_start=$results{$order}{ 's_start' };
    $s_end=$results{$order}{ 's_end' };
    $E_value=$results{$order}{ 'E_value' };
    $bit_score=$results{$order}{ 'bit_score' };


    if($current_id ne $Query_ID || $counter<10){                 #restricts the number of blast hits shown to 10 first one
	if($current_id ne $Query_ID){
	    $first=0;
	    if($i<scalar@seqnames){
	       #loop used when the sequences has no blast hits for certain sequence or $seqnames[$i] eq #cluster
               # I tried to implement this by using for-loop but couldn't make it

	       while(($i<scalar@seqnames)&&(($seqnames[$i]!~ m/$Query_ID/)||($seqnames[$i]=~ m/#cluster/))){
		     if($seqnames[$i]=~ m/#cluster/){   # The cluster markings are transmitted from this program to the next as hidden variable named seqx
                        if($query_seq_counter!=0){
			    print STDOUT "\t\t\t </td> \n";
			    print STDOUT "\t\t </tr> \n";
			    print STDOUT "\t\t <tr> \n";
                            my ($number)=$query_seq_counter;
                            print STDOUT "\t\t\t\t <td align='center' colspan='12'> <INPUT type='radio' name='combained_sequence_set$number' value='No_blast_hits'onClick=\"seqIDstorage('combained_sequence_set$number','No_blast_hits');\" >\n ";
                            print STDOUT "\t\t\t\ If you are not happy with Blast hits below , select this radio button and no blast hit is selected. This means also that no homologs can be searched from homology database.\n";
                        }
			$query_seq_counter++;
			print STDOUT "\t\t\t\t <INPUT type='hidden' name='combained_sequence_set$query_seq_counter' value='$seqnames[$i]' onClick=\"seqIDstorage('combained_sequence_set$query_seq_counter','$seqnames[$i]');\" checked>  \n ";
			$i++;
		    }
		    else{
			$query_seq_counter++;
			print STDOUT "\t\t\t </td>\n ";
                        print STDOUT "\t\t </tr>\n ";

                        if($query_seq_counter>1 && $seqnames[$i-1]!~ m/#cluster/ && $no_blast==0){
                            print STDOUT "\t\t <tr>\n ";
                            my ($number)=$query_seq_counter-1;
                            print STDOUT "\t\t\t\t <td align='center' colspan='12'> <INPUT type='radio' name='combained_sequence_set$number' value='No_blast_hits'onClick=\"seqIDstorage('combained_sequence_set$number','No_blast_hits');\" >\n ";
                            print STDOUT "\t\t\t\ If you are not happy with Blast hits below, select this radio button and no blast hit is selected. This means also that no homologs can be searched from homology database</td>\n";
                            print STDOUT "\t\t </tr>\n ";
                        }
  
                        print STDOUT "\t </table>\n";
			$no_blast=0;
                        print STDOUT "\t <br>\n";
                        print STDOUT "\t <table border='1' cellpadding='4' cellspacing='0' align='center' bordercolor='#00008B' bgcolor='#B0C4DE' width='95%' colspan='12'>\n ";
			print STDOUT "\t\t <tr>\n ";
			print STDOUT "\t\t\t <td align='center'>\n ";
			print STDOUT "\t\t\t\t There was no blast hits for '$seqnames[$i]' when given E-value was used.\n";
			print STDOUT "\t\t\t\t <INPUT type='hidden' name='combained_sequence_set$query_seq_counter' value='No_blast_hits'>  \n ";
			$i++;
			$no_blast=1;
		    }
		} 
	    }
		    $query_seq_counter++;
		    print STDOUT "\t\t\t </td>\n ";
		    print STDOUT "\t\t </tr>\n ";
		    
		    $counter=0;
                    if($query_seq_counter>1 && $seqnames[$i-1]!~ m/#cluster/ && $no_blast==0){
                         print STDOUT "\t\t <tr>\n ";
                         my ($number)=$query_seq_counter-1;
                         print STDOUT "\t\t\t\t <td align='center' colspan='12'> <INPUT type='radio' name='combained_sequence_set$number' value='No_blast_hits'onClick=\"seqIDstorage('combained_sequence_set$number','No_blast_hits');\" >\n ";
                         print STDOUT "\t\t\t\ If you are not happy with Blast hits below, select this radio button and no blast hit is selected. This means also that no homologs can be searched from homology database</td>\n";
                         print STDOUT "\t\t </tr>\n ";
                     }

		    print STDOUT "\t </table>\n";
		    $no_blast=0;
		    print STDOUT "\t <br>\n";
                    print STDOUT "\t <table border='1' cellpadding='4' cellspacing='0' align='center' bordercolor='#00008B' bgcolor='#B0C4DE' width='95%' colspan='12'>\n ";

	            print STDOUT "\t\t <tr>\n ";
		    print STDOUT "\t\t\t <td colspan='12' align='center'>\n ";
		    $print_variable=substr($Query_ID,0,28);
		    print STDOUT "\t\t\t\t Please select (only) one blast hit for query sequence $print_variable to be used as reference sequence for homology search.\n";
		    print STDOUT "\t\t\t </td>\n ";
		    print STDOUT "\t\t </tr>\n ";

                     print STDOUT "\t\t <tr>\n ";
                     print STDOUT "\t\t\t <td align='center'>Query_ID</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>Subject_ID</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>pros_identity</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>align_le</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>mismatches</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>gap_open</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>q_start</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>q_end</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>s_start</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>s_end</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>E_value</td>\n ";
                     print STDOUT "\t\t\t <td align='center'>bit_score\n";
		     $i++;
	}
	if($first==0){
	    print STDOUT "\t\t\t </td>\n";
	    print STDOUT "\t\t </tr>\n";
            print STDOUT "\t\t <tr bgcolor='#B0C4DE'>\n "; 
	    print STDOUT "\t\t\t <td align='center'> <INPUT type='radio' name='combained_sequence_set$query_seq_counter' value='$Subject_ID' onClick=\"seqIDstorage('combained_sequence_set$query_seq_counter','$Subject_ID');\" checked>\n ";
	    $first++;
	}
	else{
	    print STDOUT "\t\t\t </td>\n";
	    print STDOUT "\t\t </tr>\n";
	    print STDOUT "\t\t <tr bgcolor='#B0C4DE'>\n ";
	    print STDOUT "\t\t\t <td align='center'><INPUT type='radio' name='combained_sequence_set$query_seq_counter'  onClick=\"seqIDstorage('combained_sequence_set$query_seq_counter','$Subject_ID');\" value='$Subject_ID'>";
        }
	       $print_variable=substr($Query_ID,0,28);
               print STDOUT "$print_variable</td>\n";
               print STDOUT "\t\t\t <td align='center'>$Subject_ID</td>\n";
               print STDOUT "\t\t\t <td align='center'>$pros_identity</td>\n";
               print STDOUT "\t\t\t <td align='center'>$align_le</td>\n";
               print STDOUT "\t\t\t <td align='center'>$mismatches</td>\n";
               print STDOUT "\t\t\t <td align='center'>$gap_open</td>\n";
               print STDOUT "\t\t\t <td align='center'>$q_start</td>\n";
               print STDOUT "\t\t\t <td align='center'>$q_end</td>\n";
               print STDOUT "\t\t\t <td align='center'>$s_start</td>\n";
               print STDOUT "\t\t\t <td align='center'>$s_end</td>\n";
               print STDOUT "\t\t\t <td align='center'>$E_value</td>\n";
               print STDOUT "\t\t\t <td align='center'>$bit_score";
    }
    $counter++;
    $current_id=$Query_ID;
}
while($i<scalar@seqnames){
    $query_seq_counter++;
    print STDOUT "\t\t\t </td>\n ";
    print STDOUT "\t\t </tr>\n ";
    if($seqnames[$i]=~ m/#cluster/){             #this path is used if none of the cluster 2 sequences have any blast hits
       print STDOUT "\t\t\t\t <INPUT type='hidden' name='combained_sequence_set$query_seq_counter' value='$seqnames[$i]' onClick=\"seqIDstorage('combained_sequence_set$query_seq_counter','$seqnames[$i]');\" checked='true'> \n ";
   }
    else{
        print STDOUT "\t\t\t </td>\n ";
        print STDOUT "\t\t </tr>\n ";
	if($query_seq_counter>1 && $seqnames[$i-1]!~ m/#cluster/ && $no_blast==0){
	   print STDOUT "\t\t <tr>\n ";
	   my ($number)=$query_seq_counter-1;
                   print STDOUT "\t\t\t\t <td align='center' colspan='12'> <INPUT type='radio' name='combained_sequence_set$number' value='No_blast_hits'onClick=\"seqIDstorage('combained_sequence_set$number','No_blast_hits');\" >\n ";
                   print STDOUT "\t\t\t\ If you are not happy with Blast hits below, select this radio button and no blast hit is selected. This means also that no homologs can be searched from homology database</td>\n";
	   print STDOUT "\t\t </tr>\n ";
       }

	print STDOUT "\t </table>\n";
	print STDOUT "\t <br>\n";
        print STDOUT "\t <table border='1' cellpadding='4' cellspacing='0' align='center' bordercolor='#00008B' bgcolor='#B0C4DE' width='95%'>\n ";

	print STDOUT "\t\t <tr>\n ";
	print STDOUT "\t\t\t <td>\n ";
	print STDOUT "\t\t\t <br>\n ";
	print STDOUT "\t\t\t\t There was no blast hits for '$seqnames[$i]' when given E-value was used.\n";
	print STDOUT "\t\t\t\t <INPUT type='hidden' name='combained_sequence_set$query_seq_counter' value='No_blast_hits'>  \n ";
	$no_blast=1;
    }
    $i++;
}


print STDOUT "\t\t\t </td>\n";
print STDOUT "\t\t </tr>\n";

#sectin added 15.3.2006
	if($no_blast==0){
            print STDOUT "\t\t <tr>\n ";
            print STDOUT "\t\t\t\t <td align='center' colspan='12'> <INPUT type='radio' name='combained_sequence_set$query_seq_counter' value='No_blast_hits'onClick=\"seqIDstorage('combained_sequence_set$query_seq_counter','No_blast_hits');\" >\n ";
            print STDOUT "\t\t\t\ If you are not happy with Blast hits below, select this radio button and no blast hit is selected. This means also that no homologs can be searched from homology database</td>\n";
        }
#section added 15.3.2006 and corrected 8.6.2006
print STDOUT "\t\t </tr>\n";
print STDOUT "\t </table>\n";

print STDOUT " \t\t\t<br><br>\n ";

print STDOUT "\t<Input type=hidden name=\"input_number\" value='$nclu'>\n";
print STDOUT "\t<Input type=hidden name=\"background_organism\" value=\"$db0\">\n";
print STDOUT "\t<Input type=hidden name=\"no_query_seqs\" value=\"$amount_seq\">\n";
print STDOUT "\t<Input type=hidden name=\"search_pattern\" value=\"$pattern\">\n";
print STDOUT "\t<Input type=hidden name=\"sequence_length\" value=\"$maxlength\">\n";
print STDOUT "\t<Input type=hidden name=\"query_seqs\" value=\"$files[0]\">\n";
print STDOUT "\t<Input type=hidden name=\"input_type\" value=\"1\">\n";                  # means that input was given as sequence
print STDOUT "\t<Input type=hidden name=\"pat_overlap\" value=\"$pat_overlap\">\n";
print STDOUT "\t<Input type=hidden name=\"browser\" value=\"$browser\">\n";       


&clean_fastafile($fastafile);
unlink($output);                                      # Remove these two lines if you want to blast result files accessable from the Zope
unlink($webput);                                      # Remove these two lines if you want to blast result files accessable from the Zope

#################################################################################################################
# Prints sequences into Fasta file, counts the number of them and returns the names of sequences

sub ReadClusters {
    my($cluster_file, $maxlength1, $gff_file, $no_file) = @_; #FILE NAME, MAXIMUM LENGTH OF THE SEQUENCE, OUTPUT_FILE, NO_OF_FILE
    my($cluster) = "";                                        #CONTENT OF THE TMP-FILE
    my($line) = "";                                           #USED IN LOOPS TO READ FILES
    my(@clusterlines) = ();                                   #TMP-FILE WHICH CONTAINS SEQUENCES
    my(@sequences) = ();                                      #SEQUENCES (used for generating a fasta file for blast search)
    my(@genenames) = ();                                      #HEADERS   (used for generating a fasta file for blast search)
    my(@clusters)  = ();                                      #TABLE CONTAINING SEQUENCE NAMES AND CLUSTER INFORMATION for firsttrack.pm 
    my($fastanum) = -1;                                       #IS LINE SEQUENCE OR NOT
    my ($id)="";                                              #FASTA HEADER
    my($seq_true_len) = 0;                                    #THE TRUE LENGTH OF THE SEQUENCE
    my($no_of_seq)=0;                                         #NUMBER OF SEQUENCES IN THE FILE
    my($fasta_count)=0;                                       #added 6.3.2006

    open(IN, "<$cluster_file") or die "could not open file $cluster_file\n";
    for(<IN>) { $cluster .= $_;};
    close(IN);

   
    @clusterlines = split(/\n/, $cluster);

    foreach $line (@clusterlines) {
        if($line =~ m/^\s*$/) { next; }
        elsif ($line =~ m/^\s*#/) {
	       ++$fastanum; 
	       $fasta_count=0;                             #added 13.3.2006
	       $line =~ s/\n|\r|\t//gi;                    #modified 13.3.2006
               $line=~s/"*|>*|<*//gi;                      #added 13.3.2006

	       $clusters[$fastanum] = $line; 
	       $sequences[$fastanum] = "";                 #STORE EMPTY STRING
	       $genenames[$fastanum] = ""; 
	       next; 
	}
        elsif ($line =~ m/^\s*>/) {                         #FASTA FORMATED HEADER LINES START WITH > -CHAR
            $line =~ s/\n|\r|\t//gi;                        #REMOVE EMPTY CHARS modified 13.3.2006
	    $line=~ s/^\s*>//gi;                            #replacing the spaces with _mark
            $line=~ s/\W/_/gi;
            $line=~ s/_*$//gi;
            $line=~s/"*|>*|<*//gi;                          #added 13.3.2006

            $line=">".$line;

            ++$fastanum;                                    #WHEN THE FIRST HEADER IS FOUND c IS PUT TO ZERO
            ++$fasta_count;                                 #added 6.3.2006
	    $sequences[$fastanum] = "";                     #STORE EMPTY STRING
	    $genenames[$fastanum] = $line;                  #STORE HEADER ROW
	    $clusters[$fastanum] = $line;
	    next;
	}
	else {
	    if ($fasta_count != 0) {                        #modifications done 6.3.2006
		$line =~ s/\s//g;
                $line=~s/"*|>*|<*//gi;                      #added 13.3.2006
		$sequences[$fastanum] .= $line;
	    }
	}
    }

    unless (open (GFF, ">>$gff_file")){
	print "sorry, I could not create $gff_file";
    }

    $no_file=$no_file+1;
    my ($t)=1;
    my ($v)=1;
    my ($test)="";
    my ($test2)="";
    my ($seq_occurrency)=1;
    
    for($t=1;$t<(scalar@genenames);$t++){                                     # Takes care of the problem of same sequence given repeatedly as an input 
	$seq_occurrency=1;
        for(my($u)=0; $u<$t; $u++){ 
	    $test=substr($genenames[$t], 1, length($genenames[$t])-1);
            $test2=$genenames[$u];
            $test=~s/[\||\.|\^|\$|\*|\+|\?|\{|\[|\]|\\|\(|\)]/&/g;            #makes sure that there are no meta characters of regular expression
            $test2=~s/[\||\.|\^|\$|\*|\+|\?|\{|\[|\]|\\|\(|\)]/&/g;
            if(($test2=~m/$test$/) && ($genenames[$t] ne "")){                    #modifications done 1.3.2006
		$seq_occurrency++;
	    }
	}
	if($seq_occurrency!=1){
		#print " This is a $seq_occurrency occurrency of sequence $genenames[$t]<br>";
		$genenames[$t]=~s/>/>$seq_occurrency./;           #  (for fasta file generated for blast search)       #modifications done 2.2.2006
		#print "uusi nimi $genenames[$t]";
               
	}
    }

    $seq_occurrency=1;

    for($v=1;$v<(scalar@clusters);$v++){                                       # Takes care of the problem of same sequence given repeatedly as an input
        $seq_occurrency=1;
        for(my($x)=0; $x<$v; $x++){
            $test=substr($clusters[$v], 1, length($clusters[$v])-1);
            $test2=$clusters[$x];
            $test=~s/[\||\.|\^|\$|\*|\+|\?|\{|\[|\]|\\|\(|\)]/&/g;
            $test2=~s/[\||\.|\^|\$|\*|\+|\?|\{|\[|\]|\\|\(|\)]/&/g;

	    if(($test2=~m/$test$/) && ($clusters[$v] ne "")){        #modifications done 2.2.2006
		$seq_occurrency++;
	    }
        }
        if($seq_occurrency!=1){
                $clusters[$v]=~s/>/>$seq_occurrency./;             # (for sequence names for firsttrack.pm)  #modifications done 2.2.2006
	    }
    }





    if ($fasta_count == 0) {                                       #modifications done 6.3.2006
	print  "\n#INPUT FROM FILE $no_file DID NOT CONTAINED FASTA FORMATTED SEQUENCES, PLEASE CHECK THE FORMAT AND RESUBMIT THE JOB\n";
    }
    else{
	for(my($i) = 0; $i < scalar@sequences; ++$i) {
	    $id=$genenames[$i];
	    $id=~s/\r|\n//g;
	    $line = $sequences[$i];
	    $seq_true_len = length($line);
	    if ($maxlength1 >= $seq_true_len) {
		print GFF $genenames[$i]."\n";
		print GFF $sequences[$i]."\n\n";
	    }
	    elsif ($maxlength1 <  $seq_true_len) {
		$sequences[$i]=substr($sequences[$i],($seq_true_len-$maxlength1),$maxlength1);
		print GFF $genenames[$i]."\n";
		print GFF $sequences[$i]."\n\n";
	    }
	}
	unless (open (QUERY, ">$cluster_file")){                               #section added 13.3.2006
	    print "sorry, I could not create $cluster_file";
	}
	for(my($k) = 0; $k < scalar@clusters; ++$k) {                          #for section added 13.3.2006
	    print QUERY $clusters[$k]."\n";
	    if($sequences[$k]!~m/^$/){
		print QUERY $sequences[$k]."\n\n";
	    }
	}
	close(QUERY);
    }
     
    close (GFF);
    return \@clusters;
}
########################################################################################################################################################


sub hashValueAscendingNum {
    $results{$b} <=> $results{$a};
}

