#! /usr/bin/perl -w
use strict;
use Getopt::Long;
use vars qw($PATH);

BEGIN {
    use FindBin qw($Bin);
    use lib "$Bin";
    $PATH = $Bin;
}


use Tracklib;
use DBI;

################################ General information about this program module ###########################################################
#
# Input parameters should be given as follows:
#
#     parameter:                   Seq_IDs (file containing SEQ IDs)
#     Example file:
#                                  #C_elegans_Gene_ID
#                                  None                                               # if no sequence is found
#                                  #Fly_Gene_ID
#                                  None
#                                  #Human_Gene_ID
#                                  ENSG00000198695.1
#                                  #Mosquito_Gene_ID
#                                  None
#                                  #Mouse_Gene_ID
#                                  ENSMUSG00000064368.1
#                                  #Yeast_Gene_ID
#                                  None
#                                  #newseq                                           # denotes the beginning of new query sequence
#                                  #C_elegans_Gene_ID
#                                  None
#                                  #Fly_Gene_ID
#                                  CG12323
#                                  #Human_Gene_ID
#                                  ENSG00000100804.6
#                                  #Mosquito_Gene_ID
#                                  ENSANGG00000016059.2
#                                  #cluster                                         # this mark only when cluster changes from one to another
#                                  and so on
#
#
#     Syntax of file:               
#                                   #organism
#                                   Seq_ID1 of above organism
#                                   Seq_ID2 of above organism
#                                   #organism
#                                   Seq_ID1 of above organism
#                                   Seq_ID2 of above organism
#                                   #newseq
#                                   #organism
#                                   Seq_ID1 of above organism
#                                   Seq_ID2 of above organism
#                                   #cluster
#                                   

################################## Variables #############################################################################################

my %opt = ();
my %legend = (
              "sequence_set1=s"          =>      "file containing SEQ IDs",
	      "outfile=s"                =>      "filename for outfile",
              "webfile=s"                =>      "filename for web-file",
              "sql_user=s"               =>      "mysql user",
              "sql_pass=s"               =>      "mysql_password",
              "sql_db=s"                 =>      "mysql database",
	      "no_query_seqs=s"          =>      "number of original query sequences",
	      "search_pattern=s"         =>      "patterns searched ",
	      "sequence_length=s"         =>      "maxlength of the sequence",
	      "background_organism=s"    =>      "organism",
	      "query_seqs=s"             =>      "file containing original query sequences in fasta format",
	      "input_number=s"           =>      "number of clusters in query",
	      "seq_db=s"                 =>      "sequence db table used  for query sequences",
              "homolog_results=s"        =>      "file containing information about found homologs",
	      "pat_overlap=s"            =>      "overlapping",
	      "browser=s"                =>      "browser in use"
              );

GetOptions(\%opt, "help!", keys %legend);

my($output)                                     = "";                        # output folder
my($webput)                                     = "";                        # web address that calls me
my(%mysql_info)                                 = ();                        # hash which contains mysql parameters
my(@query)                                      = ();                        # table containing the Search IDs
my($sql_user)                                   = "";                        # Mysql username
my($sql_pass)                                   = "";                        # Mysql password
my($sql_db)                                     = "";                        # Mysql database name
my($db)                                         = "";                        # variable defining the search table from the database
my($r)                                          = 0;
my($matchkey)                                   = "";                        # Column in the database table against what the query is made
my($mysql_search)                               = "";                        # variable containing Mysql query
my(@organisms)                                  = ();                        # array containing query seach ID files
my($amount_seqs)                                = 0;                         # amount of original search sequences
my($no_organisms)                               = 0;
my(@results)                                    = ();                        # table containing results from the database query
my($result_columns)                             = "Gene, Sequence";          # returned variables from the database
my($patterns)                                   = "";                        # file containing patterns to be searched
my($s_organism)                                 = "";                        # tells which  database table to use 
my($maxlength)                                  = 0;                         # maxlength of the sequence
my($seqfile)                                    = "";                        # file containing original query sequences 
my($nclu)                                       = 0;                         # number of clusters in the query
my($queryline)                                  = "";                        # variable used in reading in the content of query_seq file
my(@queryseqs)                                  = ();                        # array containing original query sequences
my($organism)                                   = "";                        # original organism
my(@query_IDs)                                  = ();                        # table containing query IDs in case the input was given as SEQ IDs
my($q_no)=0;
my($seq_found)                                  = 0;                         # boolean variable telling whether at least one seq was found from db
my($seq_db)                                     = "";                        # sequence db table used for query sequences
my($homolog_results)                            = "";                        # file containing homology information from database.pm
my($browser)                                    = 2;
my($pat_overlap)                                 = 0;
my($call_path)					= "$Bin";
$call_path =~ s/EVOLUTION//i;
$call_path .= "VISUALIZE/visualize.pm";
#my($call_path)                                  = "$Bin/../VISUALIZE/visualize.pm";
my($query_ID_len)                               = 0;

if (exists($opt{'sequence_set1'}))       {$organisms[0] = $opt{'sequence_set1'};}  else { die "Parameter missing: sequence_set1\n"; }
if (exists($opt{'outfile'}))             {$output = $opt{'outfile'}; }             else { die "Parameter missing: outfile\n"; }
if (exists($opt{'webfile'}))             {$webput = $opt{'webfile'}; }             else { die "Parameter missing: webfile\n"; }
if (exists($opt{'sql_user'}))            {$sql_user = $opt{'sql_user'}; }          else { die "Missing parameter: sql_user"; }
if (exists($opt{'sql_pass'}))            {$sql_pass = $opt{'sql_pass'}; }          else { die "Missing parameter: sql_pass"; }
if (exists($opt{'sql_db'}))              {$sql_db = $opt{'sql_db'}; }              else { die "Missing parameter: sql_db"; }
if (exists($opt{'no_query_seqs'}))       {$amount_seqs = $opt{'no_query_seqs'};}   else { die "Missing parameter: no_query_seqs"; }
if (exists($opt{'search_pattern'}))      {$patterns = $opt{'search_pattern'};}     else { die "Missing parameter:search_pattern"; }
if (exists($opt{'sequence_length'}))     {$maxlength = $opt{'sequence_length'};}   else { die "Missing parameter: sequence_length"; }
if (exists($opt{'input_number'}))        {$nclu = $opt{'input_number'};}           else { die "Missing parameter: input_number"; }
if (exists($opt{'background_organism'})) {$organism = $opt{'background_organism'};}else { die "Missing parameter: background_organism"; }
if (exists($opt{'seq_db'}))              {$seq_db = $opt{'seq_db'}; }              else { die "Missing parameter: seq_db"; }
if (exists($opt{'query_seqs'}))          {$seqfile = $opt{'query_seqs'}; }         # not demanded, because not available when SeqID used as Tracker input
if (exists($opt{'homolog_results'}))     {$homolog_results = $opt{'homolog_results'}; }    else { die "Missing parameter: homolog_results"; }
if (exists($opt{'browser'}))             {$browser = $opt{'browser'}; }            else { die "Missing parameter: browser"; }
if (exists($opt{'pat_overlap'}))         {$pat_overlap = $opt{'pat_overlap'}; }    else { die "Missing parameter: pat_overlap"; }


$mysql_info{'sql_user'} = $sql_user;
$mysql_info{'sql_pass'} = $sql_pass;
$mysql_info{'sql_db'} = $sql_db;

$no_organisms=scalar@organisms;

for($r=0;$r<$no_organisms;$r++){
    @query=ReadModify($organisms[$r]);
}
my $no_results=scalar@query;

#for(my $x=0;$x<$no_results;$x++){
#    print "$query[$x] <br>";
# }
#print $output;


# If the tracker has been started by giving Sequences, the $seqfile contains original query sequences. They are read into table @queryseqs.


if($seqfile ne ""){
    open(IN, "<$seqfile") or die "could not open file $seqfile\n";
    for(<IN>) { $queryline .= $_;};
    close(IN);

    unlink $seqfile;                           # do not remove in case seqfile is empty.txt removed 10.2.2006
    @queryseqs = split(/\n/, $queryline);
}


#--------If the tracker has been started by giving Seq IDa as input, the Seq Ids corresponding to background orgasnism are used as "query sequences"
# They are stored into a table called @query_ID. This option is not used if there are no homologs for any of the sequences

my($homologs_found)=0;

 if(scalar@queryseqs ==0){                                                        #the check if file contains Fasta formatted query seqs 
    for(my $v=0;$v<$no_results;$v++){
	if($query[$v]=~/^#.*/){
	   if($query[$v]!~/^#cluster.*|^#newseq/){
	      $query[$v]=~/^#/;
	      $s_organism=$';
	  }
       }
	else{
	    if($s_organism eq $organism){                                        
                $query_IDs[$q_no]=$query[$v];                                    
                $q_no++;
            }
	    if($s_organism ne $organism){
		$homologs_found=1;
	    }
	}
    }
}

# if every sequence lacked the entry in homolog database, only original query seqs are transmitted to visualize.pm 

if($homologs_found==0){
    @query_IDs=();
}

#----------------------------------------------------------------------------------------------------------------------------------------------

# Opening a output file for fasta sequences, works as input file for visualize.pm

&clean_fastafile($output);
unless (open (GFF, ">>$output")){
    print "sorry, I could not create $output";
}

#################################### Database Queries ##########################################################################################
# 1. Check  -check from which database to query
# 2. Check  - check if there is a query id which to use
# 3. If Tracker has got its input as sequences, the original query sequences must be integrated into the file
# 4. If Tracker has got its input as Sequence IDs, the one from the organism used as background must be added for second time to be used as reference 

my ($i)=0;
$q_no=0;

if($seqfile ne ""){
    if((scalar@queryseqs ==0) && (scalar@query_IDs!=0)){
	print GFF "#query\n";
	$db=&definedb($organism, $seq_db);
	if($db!~m/caenorhabditis_elegans/){
	    if($query_IDs[$q_no]=~m/\./){
		$query_IDs[$q_no]=$`;
	    }
	}
        $query_ID_len=length($query_IDs[$q_no]);
        if($query_ID_len>2){
            $query_ID_len=0;
	    $mysql_search="SELECT $result_columns FROM $db WHERE Gene LIKE \"%$query_IDs[$q_no]\" \n ";  #change made 8.3.2006
	    @results=();
	    @results=@{SumSql($mysql_search,\%mysql_info, \@results)};
	    if(scalar@results!=0){
		& PrintToWebpage(\@results, $output);
		$seq_found = 1; 
	    }    
	    else{
		print GFF "#None \n";
	    }
	}
	else{
	    print GFF "#None \n";
	}
	$q_no++;
    }
    elsif(scalar@queryseqs!=0){
     $i=&PrintQuerySeq($i);
     $seq_found = 1;
    }
}


for(my $k=0;$k<$no_results;$k++){
    if($query[$k]=~/^#.*/){
	print GFF "$query[$k]\n";
	if($query[$k]!~/^#cluster.*|^#newseq/){
	    $query[$k]=~/^#/;
	    $s_organism=$';
        }
       elsif($query[$k]=~/^[#newseq.*|#cluster.*]/){
	    if((scalar@queryseqs ==0)&& (scalar@query_IDs!=0)){
		print GFF "#query\n";
		$db=&definedb($organism, $seq_db);
		if($db!~m/caenorhabditis_elegans/){
		    if($query_IDs[$q_no]=~m/\./){
			$query_IDs[$q_no]=$`;
		    }
		}
                $query_ID_len=length($query_IDs[$q_no]);
                if($query_ID_len>2){
                    $query_ID_len=0;
		    $mysql_search="SELECT $result_columns FROM $db WHERE Gene LIKE \"%$query_IDs[$q_no]\" \n "; #change made 8.3.2006
		    @results=();
		    @results=@{SumSql($mysql_search,\%mysql_info, \@results)};
		    if(scalar@results!=0){
			& PrintToWebpage(\@results, $output);
			$seq_found = 1;
		    }
		    else{
			print GFF "#None \n";
		    }
		}
		else{
		    print GFF "#None \n";
		}
		$q_no++;
	    }
	    elsif(scalar@queryseqs!=0){
		$i=&PrintQuerySeq($i);
		$seq_found = 1;
	    }
       }
    }
    else{
        if($query[$k] eq "None"){
           print GFF "#None \n";
        }
        else{
	    $db=&definedb($s_organism, $seq_db);
	    if($db!~m/caenorhabditis_elegans/){
		if($query[$k]=~m/\./){
		    $query[$k]=$`;
		}
	    }
            $query_ID_len=length($query[$k]);
            if($query_ID_len>2){
                $query_ID_len=0;
		$mysql_search="SELECT $result_columns FROM $db WHERE Gene LIKE \"%$query[$k]\" \n "; #change made 8.3.2006
		@results=();
		@results=@{SumSql($mysql_search,\%mysql_info, \@results)};
		if(scalar@results!=0){
		    & PrintToWebpage(\@results, $output);
		    $seq_found = 1;
		}
		else{
		    print GFF "#None \n";
		}
	    }
	    else{
		print GFF "#None \n";
	    }
     }
  }
}
close (GFF);


if($seq_found==1){
   &callVisualise($call_path,$output,$webput, $patterns, $nclu, $seq_db, $homolog_results,$pat_overlap, $browser);  # fourth last parameter changed from organism to seq_db
}

else{
    my ($output_html)=$output.".html";
    unless (open (VIS, ">$output_html")){
	print "sorry, I could not create $output_html";
    }
    print VIS "<dtml-var evolution_header> \n";
    print VIS "<table border cellpadding='4' cellspacing='0' align='center' bordercolor='#00008B' bgcolor='#B0C4DE' width='95%'>\n";
    print VIS "<TR>\n";
    print VIS "<TD ALIGN='center' VALIGN='top' COLSPAN='1'><font size='8'> Notification </font></TD>";
    print VIS "</TR>\n";
    print VIS "<TR>\n";
    print VIS "<TD  VALIGN='top' COLSPAN='1'><font>The Sequences corresponding to your sequence IDs could not be found from our sequence database. <br> <br>Please check the following things: <br><ul><li>1. Make sure you gave a correct file. In the file each sequence ID must lay on its own line.</li><li> 2. Please make sure that sequences provided are from the organism selected on starting page <li>3. If you are using 'clean' version, switch to 'full' version of the database</ul></font></TD>";
    print VIS "</TR>\n";
    print VIS "</table>\n";
    print VIS "<dtml-var evolution_footer>\n";
    close(VIS);
}
    unlink $patterns;
    unlink $organisms[0];

#######################################################################################################################################################

sub definedb{
    my($s_organism, $seq_db) = @_;
    my ($extension)= "_clean"; 
   
    if($seq_db=~m/_full/){
	$extension="_full";
    }
    if($s_organism eq "Fly_Gene_ID"){
	$db="drosophila_melanogaster".$extension;
    }
    elsif($s_organism eq "C_elegans_Gene_ID"){
	$db="caenorhabditis_elegans".$extension;
    }
    elsif($s_organism eq "Human_Gene_ID"){
	$db="homo_sapiens".$extension;
    }
    elsif($s_organism eq "Yeast_Gene_ID"){
	$db="saccharomyces_cerevisiae".$extension;
    }
    elsif($s_organism eq "Mouse_Gene_ID"){
	$db="mus_musculus".$extension;
    }
    elsif($s_organism eq "Mosquito_Gene_ID"){
	$db="anopheles_gambiae".$extension;
    }
    return $db;
}


######################################################################################################################################################

sub PrintToWebpage{
    my($results,$outfile) = @_;
    my $colspan=keys %{${$results}[0]};
    my $table_le= scalar(@{$results});
    my($hit) = "";
    my($value)="";
    my $ref="";
    my %hidden=();
    my $column="";


    foreach $hit (sort keys %{${$results}[0]}){
        $hidden{$hit} =$value;
      }

    for(my $l=0;$l<$table_le; $l++){
	foreach $hit (sort keys %{${$results}[$l]}){
             $value = ${${$results}[$l]}{$hit};
             if($value=~m/^$/){
		   print GFF "#None \n";
               }
               else{
		   if($hit eq "Gene"){
                      $value=">".$value;
                   }
		   print GFF "$value \n";
               }
        }
     }
}

#######################################################################################################################################################
sub PrintQuerySeq{
       if($i<(scalar@queryseqs)&& $queryseqs[$i]=~m/#/){
	  $i++;
       }
       if($i<(scalar@queryseqs-1)){
	   print GFF "#query\n";
	   $queryseqs[$i] =~ s/\n|\r//gi;
	   print GFF "$queryseqs[$i]\n";
	   $i++;
       }
    
       while($i<(scalar@queryseqs) && $queryseqs[$i]!~/^[>|#].*/){
	   $queryseqs[$i] =~ s/\n|\r//gi;
	   print GFF "$queryseqs[$i]\n";
	   $i++;
       }
    
    return $i;
}

#####################################################################################################################################################

sub callVisualise{
    my ($call_path,$input, $webinput, $patterns, $nclu, $organism, $homolog_results,$pat_overlap, $browser)=@_;
    
    system("$call_path  -num $nclu -seqfile1 $input -patfile $patterns -outfile $input -webfile $webinput -group 0 -overlap $pat_overlap -length $maxlength  -program 1 -organism $organism -homolog_results $homolog_results -browser $browser");
}
