#! /usr/bin/perl -w
use strict;

sub clean_fastafile{
    my($gff_file) = @_;
    unless (open (GFF, ">$gff_file")){
        print "sorry, I could not create $gff_file";
    }
    print GFF "";
    close (GFF);
}

#####################################################################################################################################################

sub blast_search{
    my($query_file, $database, $Evalue, $result_file)     = @_;
    my($system_flag)                        = 0;
    my ($hit)                               = "";                        # Hits from the database
    my (@hit_components)                    = ();                        # table containing information about components of each hit
    my (%query_result)                      = ();                        # hash containing blast result
    my ($order_no)                          = 1;

    # Information about 1st hit are the following:
    # $hit_components[0] -> Query id
    # $hit_components[1] -> Subject id
    # $hit_components[2] -> % identity
    # $hit_components[3] -> alignment length
    # $hit_components[4] -> mismatches
    # $hit_components[5] -> gap openings
    # $hit_components[6] -> q. start
    # $hit_components[7] -> q. end
    # $hit_components[8] -> s. start
    # $hit_components[9] -> s. end
    # $hit_components[10]-> e-value
    # $hit_components[11]-> bit score

    $system_flag =system("nice -19 blastall -e $Evalue -m 8 -p blastn -i $query_file -d $database -o $result_file");

    if ($system_flag != 0) { die "Error in Blast\n"; }
    open(IN, "<$result_file") or die "could not open file $result_file\n";
    for(<IN>) {
        $hit=$_;
        @hit_components=split(/\t/,$hit);
        $query_result{$order_no}=();
        $query_result{$order_no}{'Query_ID'}=$hit_components[0];
        $query_result{$order_no}{'Subject_ID'}=$hit_components[1];
        $query_result{$order_no}{'pros_identity'}=$hit_components[2];
        $query_result{$order_no}{'align_le'}=$hit_components[3];
        $query_result{$order_no}{'mismatches'}=$hit_components[4];
        $query_result{$order_no}{'gap_open'}=$hit_components[5];
        $query_result{$order_no}{'q_start'}=$hit_components[6];
        $query_result{$order_no}{'q_end'}=$hit_components[7];
        $query_result{$order_no}{'s_start'}=$hit_components[8];
        $query_result{$order_no}{'s_end'}=$hit_components[9];
        $query_result{$order_no}{'E_value'}=$hit_components[10];
        $query_result{$order_no}{'bit_score'}=$hit_components[11];
        $order_no++;
    }
    close(IN);
    return \%query_result;
}

######################################################################################################################################################

sub SumSql {
    my($haku, $mysql_info_ptr, $results) = @_;
    my $ref="";
    my($sql_user) = ${$mysql_info_ptr}{'sql_user'};
    my($sql_pass) = ${$mysql_info_ptr}{'sql_pass'};
    my($sql_db) = ${$mysql_info_ptr}{'sql_db'};
    my($sock) = "/var/lib/mysql/mysql.sock";                     # correction done 29.5.2008
    my($dsn) = "DBI:mysql:$sql_db;mysql_socket=$sock"; 
    my($dbh) = DBI->connect($dsn, $sql_user, $sql_pass);
    my($sth) = "";
    my($order_no)=0;

    $sth = $dbh->prepare("$haku");
    $sth->execute();


    while ($ref = $sth->fetchrow_hashref()) {
         ${$results}[$order_no]=$ref;
         $order_no++;
    }

    $dbh->disconnect();
    return $results;

}

####################################################################################################################################################

sub ReadModify{
    my($cluster_file) = @_;                         #FILE NAME
    my($cluster) = "";                              #CONTENT OF THE TMP-FILE
    my($word) = "";                                 #USED IN LOOPS TO READ FILES
    my(@clusterlines) = ();                         #TMP-FILE WHICH CONTAINS SEQUENCES
    my(@sequences) = ();                            #SEQUENCES
    my(@genenames) = ();                            #HEADERS
    my($fastanum) = -1;                             #IS LINE SEQUENCE OR NOT
    my ($id)="";                                    #FASTA HEADER
    my($seq_true_len) = 0;                          #THE TRUE LENGTH OF THE SEQUENCE


    $cluster_file=$_[0];
         

    open(IN, "<$cluster_file") or die "could not open file $cluster_file\n";
    for(<IN>) { $cluster .= $_;};
    close(IN);


    @clusterlines = split(/\n/, $cluster);
#    print "tulin Tracklib ohjelman puolelle ";
#    print @clusterlines;

    foreach $word (@clusterlines) {
        if    ($word =~ m/^\s*$/) { next; }
        else{

            $word =~ s/\s//gi;
#            if($word=~m/\./){                     # Decimal part of the Gene ID is cut away
#                $word=$`;
#           }
            ++$fastanum;
            $sequences[$fastanum] = $word;
        }
    }

    if ($fastanum == -1) {
        print  "\n#INPUT  DID NOT CONTAINED QUERY STRINGS, PLEASE CHECK THE FORMAT AND RESUBMIT THE JOB\n";
    }

    @sequences;          # RETURNS AN ARRAY CONTAINING QUERY SEQUENCES
}





1;
