#! /usr/bin/perl -w
use strict;
use Getopt::Long;
use vars qw($PATH);

BEGIN {
    use FindBin qw($Bin);
    use lib "$Bin";
    $PATH = $Bin;
}


use Tracklib;
use DBI;

################################## Variables #############################################################################################

my %opt = ();
my %legend = (
	      "seqfile1=s"                 =>      "file containing Sequence IDs",
	      "search_pattern=s"           =>      "file containing search patterns for visualize.pm",
	      "outfile=s"                  =>      "filename for saving the otput",
	      "webfile=s"                  =>      "filename for web-file",
	      "input_number=s"             =>      "parameter defining whether 1 or more clusters are used as input",
	      "sql_user=s"                 =>      "mysql user",
	      "sql_pass=s"                 =>      "mysql_password",
	      "sql_db=s"                   =>      "mysql database",
	      "background_organism=s"      =>      "database table",
	      "no_query_seqs=s"            =>      "number of query sequences",
	      "sequence_length=s"           =>      "maxlength of the sequence",
              "query_seqs=s"               =>      "file containing original query sequences",
	      "pat_overlap=s"              =>      "overlapping",
	      "browser=s"                  =>      "browser in use"
              );


GetOptions(\%opt, "help!", keys %legend);

my($output)                                     = "";                        # output folder
my($webput)                                     = "";                        # web address that calls me
my($seqidfile)                                  = "";                        # file containing SEQIDs from homology searches
my($homolog_results)                            = "";                        # file containing information about homologs found from the db
my($homolog_webresults)                         = "";                        # file containing information about homologs found from the db in zope
my($nclu)                                       = 0;                         # number of input clusters
my(%mysql_info)                                 = ();                        # hash which contains mysql parameters
my(@files)                                      = ();                        # array containing search term files
my($sql_user)                                   = "";                        # Mysql username
my($sql_pass)                                   = "";                        # Mysql password
my($sql_db)                                     = "";                        # Mysql database name
my($db)                                         = "";                        # variable defining the search table from the database (organism)
my($r)                                          = 0;
my(@totalseq)                                   = ();                        # array containing all query strings
my(@helper)                                     = ();                        # helper array needed when reading in query strings
my($matchkey)                                   = "";                        # Column in the database table against what the query is made
my($mysql_search)                               = "";                        # variable containing Mysql query
my(@results)                                    = ();                        # table containing query result
my($sql_order)                                  = "";                        # the term defining the result order
my($amount_seq)                                 = "";                        # number of query sequences
my($maxlength)                                  = 0;                         # maxlength of the sequences to be transmitted to visualize.pm 
my($seqfile)                                    = "";                        # file containing original query sequences
my($patfile)                                    = "";                        # file containing patterns for visualize.pm
my($organism)                                   = "";
my($seq_db)                                     = "";                         # tells SecondTrack.pm which db table to use for sequence retrieval
my($browser)                                    = 2;
my($pat_overlap)                                = 0;
my($call_path)                                  = "$PATH/SecondTrack.pm";     
my ($decimal)                                   = 0;

# columns retriewed from the database
my($result_columns)                             = "Human_Gene_ID, C_elegans_Gene_ID, Fly_Gene_ID, Mosquito_Gene_ID, Mouse_Gene_ID, Yeast_Gene_ID";   


if (exists($opt{'input_number'}))        { $nclu = $opt{'input_number'}; }        else { die "Parameter missing: input_number\n"; }
if (exists($opt{'seqfile1'}))            { $files[0] = $opt{'seqfile1'}; }        else { die "Parameter missing: seqfile1\n"; }
if (exists($opt{'search_pattern'}))      { $patfile = $opt{'search_pattern'};}    else { die "Parameter missing: search_pattern\n"; }
if (exists($opt{'outfile'}))             { $output = $opt{'outfile'}; }           else { die "Parameter missing: outfile\n"; }
if (exists($opt{'webfile'}))             { $webput = $opt{'webfile'}; }           else { die "Parameter missing: webfile\n"; }
if (exists($opt{'sql_user'}))            { $sql_user = $opt{'sql_user'}; }        else { die "Parameter missing: sql_user\n"; }
if (exists($opt{'sql_pass'}))            { $sql_pass = $opt{'sql_pass'}; }        else { die "Missing parameter: sql_pass"; }
if (exists($opt{'sql_db'}))              { $sql_db = $opt{'sql_db'}; }            else { die "Missing parameter: sql_db"; }
if (exists($opt{'background_organism'})) { $db = $opt{'background_organism'};}    else { die "Missing parameter: background_organism"; }
if (exists($opt{'no_query_seqs'}))       { $amount_seq = $opt{'no_query_seqs'};}  else { die "Missing parameter: no_query_seqs"; }
if (exists($opt{'sequence_length'}))      { $maxlength = $opt{'sequence_length'};}  else { die "Missing parameter: sequence_length"; }
if (exists($opt{'query_seqs'}))          { $seqfile = $opt{'query_seqs'}; }       # not demanded because not used when SEQID used as Tracker input
if (exists($opt{'browser'}))             { $browser = $opt{'browser'}; }          else { die "Missing parameter: browser"; }
if (exists($opt{'pat_overlap'}))         { $pat_overlap = $opt{'pat_overlap'}; }  else { die "Missing parameter: pat_overlap"; }


$seq_db=$db;
$seqidfile= $seqfile.".seqid.txt";              
$homolog_results= $output.".homologdb.out";
$homolog_webresults= $webput.".homologdb.out";


############################################## Transforming organism info into correct form for homology db search ####################################

if($db eq "homo_sapiens_clean" || $db eq "homo_sapiens_full"){
    $db="Homologs";
    $organism="Human_Gene_ID";
    $matchkey="Human_Gene_ID";
}
elsif($db eq "anopheles_gambiae_clean" || $db eq "anopheles_gambiae_full"){
    $db="Mosquito";
    $organism="Mosquito_Gene_ID";
    $matchkey="Mosquito_Gene_ID";
}

elsif($db eq "caenorhabditis_elegans_clean" || $db eq "caenorhabditis_elegans_full"){
    $db="Celegans";
    $organism="C_elegans_Gene_ID";
    $matchkey="C_elegans_Gene_ID";
}
elsif($db eq "drosophila_melanogaster_clean" || $db eq "drosophila_melanogaster_full"){
    $db="Dmelanogaster";
    $organism="Fly_Gene_ID";
    $matchkey="Fly_Gene_ID";
}
elsif($db eq "mus_musculus_clean" || $db eq "mus_musculus_full"){
    $db="Mouse";
    $organism="Mouse_Gene_ID";
    $matchkey="Mouse_Gene_ID";
}
elsif($db eq "saccharomyces_cerevisiae_clean" || $db eq "saccharomyces_cerevisiae_full"){
    $db="Yeast";
    $organism="Yeast_Gene_ID";
    $matchkey="Yeast_Gene_ID";
}

########################################################################################################################################################

$mysql_info{'sql_user'} = $sql_user;
$mysql_info{'sql_pass'} = $sql_pass;
$mysql_info{'sql_db'} = $sql_db;
$sql_order="ORDER BY $matchkey";

for($r=0;$r<scalar@files;$r++){
    @totalseq=&ReadModify($files[$r]);   
}

#print @totalseq;

##################################### Testing whether the database table contains decimal or whole numbers ##########################################

$mysql_search= "SELECT $matchkey FROM $db WHERE $matchkey like '%.%'";
@results=@{SumSql($mysql_search,\%mysql_info, \@results)};
if(scalar@results!=0){
    $decimal=1;
}
else{
    $decimal=0;
}

@results=();          #added 1.6.2006
####################################################################################################################################################

$mysql_search=&ReadModifySQL(\@totalseq, $matchkey, $result_columns, $db, $decimal);

if($mysql_search!~m/^$/){
    $mysql_search=$mysql_search." ".$sql_order;
    @results=@{SumSql($mysql_search,\%mysql_info, \@results)};
}

##################################### for visualizing the  database query when needed #############################################################
#each column of array contains a hash which represents one database query hit. Some query sequences may have several hits
#for (my $t=0;$t<scalar@results;$t++){
#    foreach my $hit (sort keys %{$results[$t]}){
#	print "result line $t ";
#	my $value=$results[$t]{$hit};
#	print $hit;
#	print " $value<br>";
#    }
#}
##################################################################################################################################################

unless (open (HDB, ">$homolog_results")){
    print "sorry, I could not create $output";
}

if(scalar@results!=0){
& PrintToWebpage(\@results,$call_path,$amount_seq,$seqfile, $patfile, $nclu, \@totalseq, $matchkey,$maxlength,$organism, $seq_db,$seqidfile,$output, $webput, $homolog_webresults,$pat_overlap, $browser, $decimal);
}
else{                                                                   #contains SEQ_IDs for SecondTrack.pm if db had no entries for specified SEQ ID
    &HTML_start(3);                                                     #or db search could not be made
    my $query_hidden = "";                    
    my($cluster)=1;                                                    # this may be due to fact that there was no blast_hits for any of the queries
    my($present_clu)=1;                                                # or the was no entries for specified SEQ ID even thought there was blast hits
    my($no_querys)=0;                                                   
    my($ord_quer)=1;

    print HDB "<TR>\n";
    print HDB "<TH ALIGN='center' VALIGN='top' COLSPAN='1'> Query</TH>\n";
    print HDB "<TH ALIGN='center' VALIGN='top' COLSPAN='1'> query status</TH>\n";
    print HDB "<TH ALIGN='center' VALIGN='top' COLSPAN='1'> cluster</TH>\n";
    print HDB "</TR>\n";
    
    $no_querys=scalar@totalseq;
    for(my $t=0; $t<$no_querys;$t++){
	if($totalseq[$t]=~m/^\s*#cluster/){
	   $present_clu=$cluster;
	   $cluster++;
	   if($present_clu==2){
	       $query_hidden=$query_hidden."#cluster\n";
	   }
	}
	else{
	    if($ord_quer!=1 && $totalseq[$t-1]!~m/^\s*#cluster/){   #the last sequence is needed if both clusters have only one sequence
		$query_hidden=$query_hidden."#newseq\n";
	    }
	    print HDB "<TR>\n";
	    print HDB "<TD ALIGN='center' VALIGN='top' COLSPAN='1'> Query $ord_quer</TD>\n";

	    if($totalseq[$t] ne "No_blast_hits"){
		print HDB "<TD ALIGN='center' VALIGN='top' COLSPAN='1'> Seq ID corresponding the query had no entries in our homolog database.</TD>\n";
	    }
	    else{
	        print HDB "<TD ALIGN='center' VALIGN='top' COLSPAN='1'>Query sequence had no appropriate blast hits, db search not done</TD>\n";
	    }
	    print HDB "<TD ALIGN='center' VALIGN='top' COLSPAN='1'>$present_clu</TD>\n";
	    print HDB "</TR>\n";
	    $query_hidden=$query_hidden."#".$organism."\n".$totalseq[$t]."\n";
	    $ord_quer++;
       }
    }

    &call_SecondTrack($call_path,$amount_seq,$patfile,$maxlength,$seqfile,$nclu,$organism,$query_hidden,$seq_db,$seqidfile, $output, $webput, $homolog_webresults,$pat_overlap,$browser);

}

unlink $files[0];

######################################################################################################################################################

sub ReadModifySQL{
    my($totalseq,$key, $result_columns, $db, $decimal) = @_;                     # Query ID's, SQL Query match string, SQL variables wanted, database
    my ($length)=scalar@{$totalseq};
    my($mysql_search)="";                                                        # modified 13.3.2006
    my ($first_seq)=0;

    for (my $k=0;$k<$length;$k++){
	if(($totalseq[$k]!~m/#cluster|No_blast_hits/) && ($totalseq[$k]!~m/^\s*$/) && ($totalseq[$k]!~m/\"|\%|\*|\<|\>/)){ #mod done 9.3.2006
	   if($first_seq==0){
	       $mysql_search="SELECT $result_columns FROM $db WHERE ";          # added here 13.3.2006
	       if($decimal==0 || $totalseq[$k]=~m/\./){
                   $mysql_search=$mysql_search."$key LIKE '%".$totalseq[$k]."'";
               }
               else{
		   $mysql_search=$mysql_search."$key LIKE '%".$totalseq[$k].".%'";
               }
	       $first_seq=1;
	   }
	   else{
	       if($decimal==0 || $totalseq[$k]=~m/\./){
                   $mysql_search=$mysql_search." OR $key LIKE '%".$totalseq[$k]."'";
               }
               else{
		   $mysql_search=$mysql_search." OR $key LIKE '%".$totalseq[$k].".%'";
	      }
	   }
       }
    }
  
    $mysql_search;                                                                  # RETURNS A READY MADE SQL_QUERY
}

#################################### Printing ######################################################################################################

sub PrintToWebpage{
    my($results,$call_path,$amount_seqs,$seqfile, $patfile,$nclu, $totalseq, $matchkey,$maxlength,$organism, $seq_db, $seqidfile,$output, $webput,$homolog_webresults,$pat_overlap, $browser, $decimal) = @_;
    my $colspan=keys %{${$results}[0]}; 
    my $table_le= scalar(@{$results});
    my($hit) = "";
    my($value)="";
    my($hidden)="";
    my($no_of_pats)=0;
    my(@finals)= ();                                           # Array containing the query results in most usable form
    my($query_ID)="";
    my($result_ID)="";
    my($found)=0;
    my($l)=0;
    my($cluster)=1; 
    my($present_clu)=1;
    my($place)=0;                                              #denotes into which column of table final the information is printed
    my $length=scalar@{$totalseq};
    my ($query_search)="";                                     #added 27.3.2006

    $colspan=$colspan+1;
    &HTML_start($colspan);
   
# printing the title line for the table
    print HDB "<TR>\n";
    foreach $hit (sort keys %{${$results}[0]}){
          print HDB "<TH ALIGN='center' VALIGN='top' COLSPAN='1'> $hit </TH>\n";
          for(my $m=0;$m<$length; $m++){
                  ${$finals[$m]}{$hit}="";                   # initializes hash for each query sequence
          }
    }
    print HDB "<TH ALIGN='center' VALIGN='top' COLSPAN='1'>cluster</TH>";
    for(my $l=0;$l<$length; $l++){
       ${$finals[$l]}{'cluster'}="";                   # initializes hash for each query sequence
    }

    print HDB "</TR>\n";

################################## Test printing results ###########################################################################################
#for(my ($q)=0; $q<$table_le; $q++){
#    foreach $hit (sort keys %{${$results}[$q]}){
#           print "result: $q  column: $hit value:".${${$results}[$q]}{$hit}."<br>\n";
#    }
#}
####################################################################################################################################################
# Table results contains each database search result in its own column. Each column contains a hash which contains Homolog ID's of each organism.
# This table is stored into the second table which has one column per each query sequence. Each column contains a hash containg all the homolog ID's of
# each organism corresponding to that query sequence

   for(my $k=0;$k<$length; $k++){
        $found=0;
        $query_ID=${$totalseq}[$k];
        if($decimal==0 ||$query_ID=~m/\./){                 #added 28.3.2006
	    $query_search=$query_ID."\$";                   #added 27.3.2006
	}                                                   #added 27.3.2006
        else{                                               #added 27.3.2006
	    $query_search=$query_ID.".";                    #added 27.3.2006
	}                                                   #added 27.3.2006


        if($query_ID=~m/^\s*#cluster/){
	   $present_clu=$cluster;
	   $cluster++;
           $place--;                      
         }
        else{
	    for( $l=0;$l<$table_le; $l++){
                $result_ID=${${$results}[$l]}{$matchkey};
#      		if($result_ID=~m/\./){                     # Decimal part of the Gene ID is cut away
#		   $result_ID=$`;
#		}
                 if($result_ID=~m/$query_search/){         #added 27.3.2006      
#                if($query_ID=~m/$result_ID/){             # fixed 13.1.2006 
		    $found++;
		    ${$finals[$place]}{'cluster'}=$present_clu;
		    foreach $hit (sort keys %{${$results}[$l]}){
                            # preventing the same SEQ Id from occurring twice
 		            if((${${$results}[$l]}{$hit}!~m/^$/) &&(${$finals[$place]}{$hit}!~${${$results}[$l]}{$hit})){ 
                                ${$finals[$place]}{$hit} =${$finals[$place]}{$hit}."\n".${${$results}[$l]}{$hit};
                            }

	            }
               }
            }
            if($found==0){
		my $seq=$place+1;
#		print "The Sequence ID corresponding the query sequence $seq has no entries in our homolog database\n<br>";
		${$finals[$place]}{'cluster'}=$present_clu;
                ${$finals[$place]}{'query'}="\n".$query_ID;                        #added 4.1.2006
	    }
       }
       $place++;
    }

######################################## Printing the generated @finals array ################################################################
#for(my $n=0;$n<$amount_seq; $n++){
#    foreach $hit (sort keys %{$finals[$n]}){
#	$value = ${$finals[$n]}{$hit};
#        print "$value <br>";
#    }
#}
#############################################################################################################################################


# The Final data sturcture is ready containing each query sequences homolog information in the hash in the corresponding table value
# The information is printed also into the  table and into the hidden input values on the html- page
     my ($newcluster)= " ";

     for(my $n=0;$n<$amount_seq; $n++){
        print HDB "<TR>\n";
	$value = ${$finals[$n]}{'cluster'};

        if($newcluster ne $value){
	    if($n!=0){
		my($end_of_hidden)=substr($hidden,(length($hidden)-8),8);
                if($end_of_hidden=~m/^\s*#newseq/){                            # this is because if #cluster defining new cluster is printed there is 
		   $hidden=substr($hidden,0,(length($hidden)-8));              # probably no need to underline that also new seq has began    
                }
		$hidden=$hidden."#cluster\n";
	    }
	    $newcluster=$value;
        }
        foreach $hit (sort keys %{$finals[$n]}){
	    if($hit ne 'query'){
	   	$value = ${$finals[$n]}{$hit};
                if($value=~m/^$/){
		    if($hit eq $organism){
			my ($q_no)=$n+1;
			if(${$finals[$n]}{'query'}=~m/No_blast_hits/){
			   print HDB "<TD ALIGN='center' VALIGN='top' COLSPAN='1'>Query sequence $q_no had no appropriate blast hits, db search not done</TD>\n";
			   $hidden=$hidden."#".$hit."\n"."None\n";
		        }
		        else{
			    print HDB "<TD ALIGN='center' VALIGN='top' COLSPAN='1'>Query sequence $q_no has no entries in homology database</TD>\n";
			    # in case that query sequence is not found from homolog db the empty value is replaced by original query ID
			    $hidden=$hidden."#".$hit.${$finals[$n]}{'query'}."\n";
		       }
		    }
		    else{
			print HDB "<TD ALIGN='center' VALIGN='top' COLSPAN='1'>-</TD>\n";
		        if($hit ne 'cluster'){
			    $hidden=$hidden."#".$hit."\n"."None\n";
			}
		    }
                }
                else{
                   print HDB "<TD ALIGN='center' VALIGN='top' COLSPAN='1'> $value</TD>\n";
		   if($hit ne 'cluster'){
		       $hidden=$hidden."#".$hit.$value."\n";
		   }
                }
            }
        }
        if($n!=$amount_seq-1){
	    $hidden=$hidden."#newseq\n";
        }
        print HDB "</TR>\n";

     }
     &call_SecondTrack($call_path,$amount_seqs,$patfile,$maxlength,$seqfile,$nclu,$organism,$hidden,$seq_db, $seqidfile, $output, $webput,  $homolog_webresults,$pat_overlap, $browser);
     print HDB "</table>\n";
     print HDB "</body> \n </html>\n";
     close(HDB);

}

#####################################################################################################################################################

sub HTML_start{
    my($colspan) = @_;
    print HDB "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">\n";
    print HDB "<html>\n<body>\n";
    print HDB "<table border cellpadding='4' cellspacing='0' align='center' bordercolor='#00008B' bgcolor='#B0C4DE' width='95%'>\n";
    print HDB "<TR>\n";
    print HDB "<TD ALIGN='center' VALIGN='top' COLSPAN='$colspan'><font><b>Information</b></font></TD>\n";
    print HDB "</TR>\n";
}

###################################################################################################################################################

sub call_SecondTrack{
    my($call_path,$amount_seqs,$patfile,$maxlength,$seqfile,$nclu,$organism,$hidden,$seq_db, $seqidfile, $output, $webput,  $homolog_webresults, $pat_overlap, $browser) = @_;


#---------------- Storing the found sequence IDs into file ---------------------------------------------------------------------------------------

    unless (open (ID, ">$seqidfile")){
	print "sorry, I could not create $seqidfile";
    }
    print ID "$hidden";
    close($seqidfile);
     
    
#--------------- Calling program SecondTrack.pm -------------------------------------------------------------------------------------------------

 system("$call_path  -sequence_set1 $seqidfile -outfile $output -webfile $webput -sql_user matti -sql_pass matti -sql_db matti -no_query_seqs $amount_seqs -search_pattern $patfile -sequence_length $maxlength -background_organism $organism -query_seqs $seqfile -input_number $nclu -seq_db $seq_db -homolog_results  $homolog_webresults -pat_overlap $pat_overlap -browser $browser");
         
}
